﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;

namespace HeuristicLab.Algorithms.CMAEvolutionStrategy {
  [Item("CMAAnalyzer", "Analyzes the development of strategy parameters and visualizes the performance of CMA-ES.")]
  [StorableClass]
  public sealed class CMAAnalyzer : SingleSuccessorOperator, IAnalyzer {

    public bool EnabledByDefault {
      get { return false; }
    }

    #region Parameter Properties
    public ILookupParameter<CMAParameters> StrategyParametersParameter {
      get { return (ILookupParameter<CMAParameters>)Parameters["StrategyParameters"]; }
    }

    public ILookupParameter<RealVector> MeanParameter {
      get { return (ILookupParameter<RealVector>)Parameters["Mean"]; }
    }

    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    #endregion

    [StorableConstructor]
    private CMAAnalyzer(bool deserializing) : base(deserializing) { }
    private CMAAnalyzer(CMAAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public CMAAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<CMAParameters>("StrategyParameters", "The CMA strategy parameters to be analyzed."));
      Parameters.Add(new LookupParameter<RealVector>("Mean", "The mean real vector that is being optimized."));
      Parameters.Add(new LookupParameter<ResultCollection>("Results", "The collection to store the results in."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CMAAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var sp = StrategyParametersParameter.ActualValue;
      var vector = MeanParameter.ActualValue;
      var results = ResultsParameter.ActualValue;

      DataTable axisRatio;
      if (results.ContainsKey("AxisRatio")) {
        axisRatio = (DataTable)results["AxisRatio"].Value;
      } else {
        axisRatio = new DataTable("AxisRatio");
        axisRatio.Rows.Add(new DataRow("AxisRatio"));
        axisRatio.VisualProperties.YAxisLogScale = true;
        results.Add(new Result("AxisRatio", axisRatio));
      }
      axisRatio.Rows["AxisRatio"].Values.Add(sp.AxisRatio.Value);

      DataTable sigma;
      if (results.ContainsKey("Sigma")) {
        sigma = (DataTable)results["Sigma"].Value;
      } else {
        sigma = new DataTable("Sigma");
        sigma.VisualProperties.YAxisLogScale = true;
        sigma.Rows.Add(new DataRow("Sigma"));
        results.Add(new Result("Sigma", sigma));
      }
      sigma.Rows["Sigma"].Values.Add(sp.Sigma.Value);

      DataTable scaling;
      if (results.ContainsKey("Scaling")) {
        scaling = (DataTable)results["Scaling"].Value;
      } else {
        scaling = new DataTable("Scaling");
        scaling.VisualProperties.YAxisLogScale = true;
        for (int i = 0; i < sp.C.Rows; i++)
          scaling.Rows.Add(new DataRow("Axis" + i.ToString()));
        results.Add(new Result("Scaling", scaling));
      }
      for (int i = 0; i < sp.C.Rows; i++)
        scaling.Rows["Axis" + i.ToString()].Values.Add(sp.D[i]);

      DataTable realVector;
      if (results.ContainsKey("RealVector")) {
        realVector = (DataTable)results["RealVector"].Value;
      } else {
        realVector = new DataTable("RealVector");
        for (int i = 0; i < vector.Length; i++)
          realVector.Rows.Add(new DataRow("Axis" + i.ToString()));
        results.Add(new Result("RealVector", realVector));
      }
      for (int i = 0; i < vector.Length; i++)
        realVector.Rows["Axis" + i.ToString()].Values.Add(vector[i]);

      DataTable stdDevs;
      if (results.ContainsKey("StandardDeviations")) {
        stdDevs = (DataTable)results["StandardDeviations"].Value;
      } else {
        stdDevs = new DataTable("StandardDeviations");
        stdDevs.VisualProperties.YAxisLogScale = true;
        for (int i = 0; i < vector.Length; i++)
          stdDevs.Rows.Add(new DataRow("Axis" + i.ToString()));
        results.Add(new Result("StandardDeviations", stdDevs));
      }
      for (int i = 0; i < vector.Length; i++)
        stdDevs.Rows["Axis" + i.ToString()].Values.Add(Math.Sqrt(sp.C[i, i]));

      return base.Apply();
    }
  }
}